#!/usr/bin/perl
use strict;
use warnings;
use Test::More qw(no_plan);

use Data::Dumper;
use RDF::Redland;
use RDF::Query;
use Geo::Distance;
use Encode;

use Location::Name;

my @data	= map { RDF::Redland::URI->new( 'file://' . File::Spec->rel2abs( "data/$_" ) ) } qw(about.xrdf foaf.xrdf);
my $storage	= new RDF::Redland::Storage("hashes", "test", "new='yes',hash-type='memory'");
my $model	= new RDF::Redland::Model($storage, "");
my $parser	= new RDF::Redland::Parser("rdfxml");
$parser->parse_into_model($_, $_, $model) for (@data);

{
	my $sparql	= <<"END";
		PREFIX	rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
		PREFIX	foaf: <http://xmlns.com/foaf/0.1/>
		PREFIX	dcterms: <http://purl.org/dc/terms/>
		PREFIX	geo: <http://www.w3.org/2003/01/geo/wgs84_pos#>
		PREFIX	mygeo: <http://kasei.us/e/ns/geo#>
		SELECT	?image ?point ?name ?lat ?long
		WHERE	{
					?image rdf:type foaf:Image .
					?image dcterms:spatial ?point .
					?point foaf:name ?name .
					?point geo:lat ?lat .
					?point geo:long ?long .
					FILTER mygeo:distance(?point, 41.849331, -71.392) < 10 .
				}
END
	my $query	= RDF::Query->new( $sparql, undef, undef, 'sparql' );
	$query->add_function( 'http://kasei.us/e/ns/geo#distance', sub {
		my $geo			= new Geo::Distance;
		my $point		= shift;
		my $plat		= get_first_literal( $model, $point, 'http://www.w3.org/2003/01/geo/wgs84_pos#lat' );
		my $plon		= get_first_literal( $model, $point, 'http://www.w3.org/2003/01/geo/wgs84_pos#long' );
		my $lookup	= new Location::Name;
		my $name	= $lookup->name( $plat, $plon );
		# warn "POINT: $name ($plat, $plon)\n";
		my ($lat, $lon)	= @_;
		my $dist	= $geo->distance(
						unit => 'kilometer',
						lon1 => $lon,
						lat1 => $lat,
						lon2 => $plon,
						lat2 => $plat
					);
		# warn "\t-> ${dist} kilometers from Providence";
		return $dist;
	} );
	my $stream	= $query->execute( $model );
	while (my $row = $stream->()) {
		my ($image, $point, $pname, $lat, $lon)	= @{ $row };
		my $url		= $image->uri->as_string;
		my $name	= $pname->literal_value;
		like( $name, qr/, (RI|MA|CT)$/, "$name ($url)" );
	}
}


######################################################################

sub get_first_literal {
	my $node	= get_first_obj( @_ );
	return $node ? decode('utf8', $node->literal_value) : undef;
}

sub get_first_obj {
	my $model	= shift;
	my $node	= shift;
	my $uri		= shift;
	my @uris	= UNIVERSAL::isa($uri, 'ARRAY') ? @{ $uri } : ($uri);
	my @preds	= map { ref($_) ? $_ : RDF::Redland::Node->new_from_uri( $_ ) } @uris;
	foreach my $pred (@preds) {
		my $targets	= $model->targets_iterator( $node, $pred );
		while ($targets and !$targets->end) {
			my $node	= $targets->current;
			return $node if ($node);
		} continue { $targets->next }
	}
}
